package com.haomostudio.SeekTruthBackend.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.haomostudio.SeekTruthBackend.common.Resp;
import com.haomostudio.SeekTruthBackend.service.CatalogService;
import com.haomostudio.SeekTruthBackend.service.HmUtils.Tools;
import com.haomostudio.delivery.po.Catalog;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "菜单表")
@Controller
public class CatalogController {

    @Autowired
    CatalogService catalogService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'菜单表'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/catalogs/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createCatalog(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "cid", required = false) String cid,
            @RequestParam(value = "cname", required = false) String cname,
            @RequestParam(value = "resource", required = false) String resource,
            @RequestParam(value = "createTime", required = false) String createTime,
            @RequestParam(value = "lastUpdateTime", required = false) String lastUpdateTime
    ) {
        Catalog item = new Catalog();
        String id = Tools.getUUID();
        item.setId(id);
        if( cid != null ){
            item.setCid(cid);
        }
        if( cname != null ){
            item.setCname(cname);
        }
        if( resource != null ){
            item.setResource(resource);
        }
            item.setCreateTime(new Date());
            item.setLastUpdateTime(new Date());
        catalogService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return catalogService.get(id);
        //return item;
    }

    @ApiOperation(value = "删除'菜单表'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/catalogs/{catalog_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteCatalog(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "catalog_id") String id
    ) {
        Catalog item = catalogService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        catalogService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'菜单表'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/catalogs/{catalog_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editCatalog(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "catalog_id") String id,
            @RequestParam(value = "cid", required = false) String cid,
            @RequestParam(value = "cname", required = false) String cname,
            @RequestParam(value = "resource", required = false) String resource
    ) {
        Catalog item = catalogService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( cid != null ){
            item.setCid(cid);
        }
        if( cname != null ){
            item.setCname(cname);
        }
        if( resource != null ){
            item.setResource(resource);
        }
            item.setLastUpdateTime(new Date());
        catalogService.update(item);

        return catalogService.get(id);
    }

    @ApiOperation(value = "查询'菜单表'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/catalogs/{catalog_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getCatalog(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "catalog_id") String id,
            @RequestParam(value = "cid", required = false) String cid,
            @RequestParam(value = "cname", required = false) String cname,
            @RequestParam(value = "resource", required = false) String resource
    ) {
        if(request.getMethod().equals("GET")){
            Catalog item = catalogService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            Catalog item = catalogService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( cid != null ){
                item.setCid(cid);
            }
            if( cname != null ){
                item.setCname(cname);
            }
            if( resource != null ){
                item.setResource(resource);
            }
                item.setLastUpdateTime(new Date());
            catalogService.update(item);

            return catalogService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            Catalog item = catalogService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            catalogService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'菜单表'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/catalogs",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getCatalogs(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = catalogService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return catalogService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/catalogs/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getCatalogsId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<Catalog> list = catalogService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = Catalog.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'菜单表'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/catalogs/create/batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsCatalogs(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<Catalog> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             Catalog item = new Catalog();
             String id = Tools.getUUID();
                         item.setId(id);
             if(jsonObject.containsKey("cid")   ){
            item.setCid(jsonObject.get("cid").toString());
        }
        if(jsonObject.containsKey("cname")   ){
            item.setCname(jsonObject.get("cname").toString());
        }
        if(jsonObject.containsKey("resource")   ){
            item.setResource(jsonObject.get("resource").toString());
        }
            item.setCreateTime(new Date());
            item.setLastUpdateTime(new Date());
             catalogService.create(item);
             lists.add(catalogService.get(id));

             });
        return lists;
     }

     @ApiOperation(value = "批量修改'菜单表'表中多条信息", notes = "通过post方法请求，传入表中字段的对应信息，数组中每个对象必须有id属性！")
     @RequestMapping(value = "/catalogs/edit/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
     @ResponseBody
     public Object editsCatalogs(
              HttpServletRequest request,
              @RequestHeader(value = "X-Auth-Token", required = false) String token,
              @RequestParam(value = "params", required = false) String params
     ) {

          JSONArray jsonArray = JSON.parseArray(params);
          List<Catalog> lists  = new ArrayList<>();

          jsonArray.stream().forEach((Object object) -> {

              JSONObject jsonObject = JSONObject.parseObject(object.toString());

              Catalog item = new Catalog();

              if(jsonObject.containsKey("cid")   ){
            item.setCid(jsonObject.get("cid").toString());
        }
        if(jsonObject.containsKey("cname")   ){
            item.setCname(jsonObject.get("cname").toString());
        }
        if(jsonObject.containsKey("resource")   ){
            item.setResource(jsonObject.get("resource").toString());
        }
        if(jsonObject.containsKey("id")   ){
            item.setId(jsonObject.get("id").toString());
        }
            item.setLastUpdateTime(new Date());
              catalogService.update(item);
              lists.add(catalogService.get(jsonObject.get("id").toString()));

          });
         return lists;
      }



    @ApiOperation(value = "批量删除'菜单表'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/catalogs/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesCatalog(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 Catalog item = catalogService.get(id.toString());
                if (null != item) {
                    catalogService.delete(id.toString());
                }

            });
            return Resp.succ("delete success");
        }
}
