package com.haomostudio.SeekTruthBackend.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.haomostudio.SeekTruthBackend.common.Resp;
import com.haomostudio.SeekTruthBackend.service.HmUtils.Tools;
import com.haomostudio.SeekTruthBackend.service.UserService;
import com.haomostudio.delivery.po.User;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "用户表")
@Controller
public class UserController {

    @Autowired
    UserService userService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'用户表'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/users/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createUser(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "uid", required = false) String uid,
            @RequestParam(value = "username", required = false) String username,
            @RequestParam(value = "did", required = false) String did,
            @RequestParam(value = "password", required = false) String password,
            @RequestParam(value = "gender", required = false) String gender,
            @RequestParam(value = "age", required = false) String age,
            @RequestParam(value = "createTime", required = false) String createTime,
            @RequestParam(value = "lastUpdateTime", required = false) String lastUpdateTime
    ) {
        User item = new User();
        String id = Tools.getUUID();
        item.setId(id);
        if( uid != null ){
            item.setUid(uid);
        }
        if( username != null ){
            item.setUsername(username);
        }
        if( did != null ){
            item.setDid(did);
        }
        if( password != null ){
            item.setPassword(password);
        }
        if( gender != null ){
            item.setGender(gender);
        }
        if( age != null ){
            item.setAge(age);
        }
            item.setCreateTime(new Date());
            item.setLastUpdateTime(new Date());
        userService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return userService.get(id);
        //return item;
    }

    @ApiOperation(value = "删除'用户表'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/users/{user_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteUser(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "user_id") String id
    ) {
        User item = userService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        userService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'用户表'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/users/{user_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editUser(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "user_id") String id,
            @RequestParam(value = "uid", required = false) String uid,
            @RequestParam(value = "username", required = false) String username,
            @RequestParam(value = "did", required = false) String did,
            @RequestParam(value = "password", required = false) String password,
            @RequestParam(value = "gender", required = false) String gender,
            @RequestParam(value = "age", required = false) String age
    ) {
        User item = userService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( uid != null ){
            item.setUid(uid);
        }
        if( username != null ){
            item.setUsername(username);
        }
        if( did != null ){
            item.setDid(did);
        }
        if( password != null ){
            item.setPassword(password);
        }
        if( gender != null ){
            item.setGender(gender);
        }
        if( age != null ){
            item.setAge(age);
        }
            item.setLastUpdateTime(new Date());
        userService.update(item);

        return userService.get(id);
    }

    @ApiOperation(value = "查询'用户表'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/users/{user_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getUser(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "user_id") String id,
            @RequestParam(value = "uid", required = false) String uid,
            @RequestParam(value = "username", required = false) String username,
            @RequestParam(value = "did", required = false) String did,
            @RequestParam(value = "password", required = false) String password,
            @RequestParam(value = "gender", required = false) String gender,
            @RequestParam(value = "age", required = false) String age
    ) {
        if(request.getMethod().equals("GET")){
            User item = userService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            User item = userService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( uid != null ){
                item.setUid(uid);
            }
            if( username != null ){
                item.setUsername(username);
            }
            if( did != null ){
                item.setDid(did);
            }
            if( password != null ){
                item.setPassword(password);
            }
            if( gender != null ){
                item.setGender(gender);
            }
            if( age != null ){
                item.setAge(age);
            }
                item.setLastUpdateTime(new Date());
            userService.update(item);

            return userService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            User item = userService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            userService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'用户表'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/users",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getUsers(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = userService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return userService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/users/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getUsersId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<User> list = userService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = User.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'用户表'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/users/create/batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsUsers(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<User> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             User item = new User();
             String id = Tools.getUUID();
                         item.setId(id);
             if(jsonObject.containsKey("uid")   ){
            item.setUid(jsonObject.get("uid").toString());
        }
        if(jsonObject.containsKey("username")   ){
            item.setUsername(jsonObject.get("username").toString());
        }
        if(jsonObject.containsKey("did")   ){
            item.setDid(jsonObject.get("did").toString());
        }
        if(jsonObject.containsKey("password")   ){
            item.setPassword(jsonObject.get("password").toString());
        }
        if(jsonObject.containsKey("gender")   ){
            item.setGender(jsonObject.get("gender").toString());
        }
        if(jsonObject.containsKey("age")   ){
            item.setAge(jsonObject.get("age").toString());
        }
            item.setCreateTime(new Date());
            item.setLastUpdateTime(new Date());
             userService.create(item);
             lists.add(userService.get(id));

             });
        return lists;
     }

     @ApiOperation(value = "批量修改'用户表'表中多条信息", notes = "通过post方法请求，传入表中字段的对应信息，数组中每个对象必须有id属性！")
     @RequestMapping(value = "/users/edit/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
     @ResponseBody
     public Object editsUsers(
              HttpServletRequest request,
              @RequestHeader(value = "X-Auth-Token", required = false) String token,
              @RequestParam(value = "params", required = false) String params
     ) {

          JSONArray jsonArray = JSON.parseArray(params);
          List<User> lists  = new ArrayList<>();

          jsonArray.stream().forEach((Object object) -> {

              JSONObject jsonObject = JSONObject.parseObject(object.toString());

              User item = new User();

              if(jsonObject.containsKey("uid")   ){
            item.setUid(jsonObject.get("uid").toString());
        }
        if(jsonObject.containsKey("username")   ){
            item.setUsername(jsonObject.get("username").toString());
        }
        if(jsonObject.containsKey("did")   ){
            item.setDid(jsonObject.get("did").toString());
        }
        if(jsonObject.containsKey("password")   ){
            item.setPassword(jsonObject.get("password").toString());
        }
        if(jsonObject.containsKey("gender")   ){
            item.setGender(jsonObject.get("gender").toString());
        }
        if(jsonObject.containsKey("age")   ){
            item.setAge(jsonObject.get("age").toString());
        }
        if(jsonObject.containsKey("id")   ){
            item.setId(jsonObject.get("id").toString());
        }
            item.setLastUpdateTime(new Date());
              userService.update(item);
              lists.add(userService.get(jsonObject.get("id").toString()));

          });
         return lists;
      }



    @ApiOperation(value = "批量删除'用户表'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/users/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesUser(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 User item = userService.get(id.toString());
                if (null != item) {
                    userService.delete(id.toString());
                }

            });
            return Resp.succ("delete success");
        }
}
